#include "util-linux-automotive.h"

#include <blkid/blkid.h>
#include <string>
#include <unistd.h>

#define autoblkid_dev_iterate_end __attribute__ ((cleanup (cleanup_blkid_dev_iterate_end)))
#define autoblkid_put_cache __attribute__ ((cleanup (cleanup_blkid_put_cache)))

static inline void
cleanup_blkid_dev_iterate_end (blkid_dev_iterate *p)
{
  blkid_dev_iterate_end (*p);
}

static inline void
cleanup_blkid_put_cache (blkid_cache *p)
{
  blkid_put_cache (*p);
}

static int
each_block_device (blkid_dev *dev, const blkid_cache &cache)
{
  const char *devname = blkid_dev_devname (*dev);
  if (devname)
    {
      autofree char *partlabel = blkid_get_tag_value (cache, "PARTLABEL", devname);
      if (partlabel)
        {
          std::string partlabel_escaped = "/dev/disk/by-partlabel/";
          for (size_t i = 0; partlabel[i]; ++i)
            if (partlabel[i] == ' ')
              partlabel_escaped += "\\x20";
            else
              partlabel_escaped += partlabel[i];

          const std::string devname_final = "../../" + std::string (basename (devname));
          debug ("symlink (\"%s\", \"%s\")", devname_final.c_str (), partlabel_escaped.c_str ());
          if (symlink (devname_final.c_str (), partlabel_escaped.c_str ()) < 0)
            {
              perrorf ("symlink (\"%s\", \"%s\")", devname_final.c_str (),
                       partlabel_escaped.c_str ());
              return 4;
            }
        }
    }

  return 0;
}

int
main (int argc, char *[])
{
  if (argc > 1)
    enable_debug = true;

  autoblkid_put_cache blkid_cache cache;
  blkid_dev dev;

  // Initialize the blkid cache
  if (blkid_get_cache (&cache, nullptr) < 0)
    {
      perror ("blkid_get_cache");
      return 1;
    }

  if (blkid_probe_all (cache) < 0)
    {
      perror ("blkid_probe_all");
      return 2;
    }

  // Initialize the iterator
  autoblkid_dev_iterate_end blkid_dev_iterate iterator = blkid_dev_iterate_begin (cache);
  if (!iterator)
    {
      perror ("blkid_dev_iterate_begin");
      return 3;
    }

  // Iterate over all block devices
  while (blkid_dev_next (iterator, &dev) == 0)
    {
      const int ret = each_block_device (&dev, cache);
      if (ret)
        return ret;
    }

  return 0;
}
